;; Generated package description from ebdb-gnorb.el  -*- no-byte-compile: t -*-
(define-package "ebdb-gnorb" "1.0.2" "Utilities for connecting EBDB to Gnorb" '((gnorb "1.1.0") (ebdb "0.2")) :commit "461a6b35dc9322d1ec59547ad845d26a6c65a698" :url "https://elpa.gnu.org/packages/ebdb-gnorb.html" :authors '(("Eric Abrahamsen" . "eric@ericabrahamsen.net")) :maintainer '("Eric Abrahamsen" . "eric@ericabrahamsen.net"))
